@@ -166,7 +166,7 @@ def _retry_read_url(url, retry_count, pause, name):
             pass
         else:
             rs = read_csv(StringIO(bytes_to_str(lines)), index_col=0,
-                          parse_dates=True)[::-1]
+                          parse_dates=True, na_values='-')[::-1]
             # Yahoo! Finance sometimes does this awesome thing where they
             # return 2 rows for the most recent business day
             if len(rs) > 2 and rs.index[-1] == rs.index[-2]:  # pragma: no cover
@@ -685,8 +685,14 @@ class Options(object):
             except IndexError:
                 self.underlying_price, self.quote_time = np.nan, np.nan
 
-        calls = self._process_data(frames[self._TABLE_LOC['calls']], 'call')
-        puts = self._process_data(frames[self._TABLE_LOC['puts']], 'put')
+        calls = frames[self._TABLE_LOC['calls']]
+        puts = frames[self._TABLE_LOC['puts']]
+
+        if len(calls) == 0 or len(puts) == 0:
+            raise RemoteDataError('Received no data from Yahoo at url: %s' % url)
+
+        calls = self._process_data(calls, 'call')
+        puts = self._process_data(puts, 'put')
 
         return {'calls': calls, 'puts': puts}
 
@@ -1132,10 +1138,13 @@ class Options(object):
 
         expiry_dates = [dt.datetime.strptime(element.text, "%B %d, %Y").date() for element in links]
         links = [element.attrib['data-selectbox-link'] for element in links]
+
+        if len(expiry_dates) == 0:
+            raise RemoteDataError('Data not available')
+
         expiry_links = dict(zip(expiry_dates, links))
         self._expiry_links = expiry_links
         self._expiry_dates = expiry_dates
-
         return expiry_dates, expiry_links
 
     def _parse_url(self, url):
