@@ -411,15 +411,19 @@ def _get_range_edges(first, last, offset, closed='left', base=0):
 def _adjust_dates_anchored(first, last, offset, closed='right', base=0):
     from pandas.tseries.tools import normalize_date
 
+    # First and last offsets should be calculated from the start day to fix an
+    # error cause by resampling across multiple days when a one day period is
+    # not a multiple of the frequency.
+    #
+    # See https://github.com/pydata/pandas/issues/8683
+
     start_day_nanos = Timestamp(normalize_date(first)).value
-    last_day_nanos = Timestamp(normalize_date(last)).value
 
     base_nanos = (base % offset.n) * offset.nanos // offset.n
     start_day_nanos += base_nanos
-    last_day_nanos += base_nanos
 
     foffset = (first.value - start_day_nanos) % offset.nanos
-    loffset = (last.value - last_day_nanos) % offset.nanos
+    loffset = (last.value - start_day_nanos) % offset.nanos
 
     if closed == 'right':
         if foffset > 0:
