@@ -24,7 +24,7 @@ import numpy as np
 from numpy.testing import assert_array_equal
 
 import pandas as pd
-from pandas.core.common import _is_sequence, array_equivalent
+from pandas.core.common import is_sequence, array_equivalent, is_list_like
 import pandas.core.index as index
 import pandas.core.series as series
 import pandas.core.frame as frame
@@ -72,13 +72,15 @@ def reset_testing_mode():
     if 'deprecate' in testing_mode:
         warnings.simplefilter('ignore', DeprecationWarning)
 
+
 set_testing_mode()
 
+
 class TestCase(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        pd.set_option('chained_assignment','raise')
+        pd.set_option('chained_assignment', 'raise')
 
     @classmethod
     def tearDownClass(cls):
@@ -86,19 +88,7 @@ class TestCase(unittest.TestCase):
 
     def reset_display_options(self):
         # reset the display options
-        pd.reset_option('^display.',silent=True)
-
-    def assert_numpy_array_equal(self, np_array, assert_equal):
-        """Checks that 'np_array' is equal to 'assert_equal'
-
-        Note that the expected array should not contain `np.nan`! Two numpy arrays are equal if all
-        elements are equal, which is not possible if `np.nan` is such an element!
-
-        If the expected array includes `np.nan` use `assert_numpy_array_equivalent(...)`.
-        """
-        if np.array_equal(np_array, assert_equal):
-            return
-        raise AssertionError('{0} is not equal to {1}.'.format(np_array, assert_equal))
+        pd.reset_option('^display.', silent=True)
 
     def round_trip_pickle(self, obj, path=None):
         if path is None:
@@ -107,82 +97,6 @@ class TestCase(unittest.TestCase):
             pd.to_pickle(obj, path)
             return pd.read_pickle(path)
 
-    def assert_numpy_array_equivalent(self, np_array, assert_equal, strict_nan=False):
-        """Checks that 'np_array' is equivalent to 'assert_equal'
-
-        Two numpy arrays are equivalent if the arrays have equal non-NaN elements, and
-        `np.nan` in corresponding locations.
-
-        If the the expected array does not contain `np.nan` `assert_numpy_array_equivalent` is the
-        similar to `assert_numpy_array_equal()`. If the expected array includes `np.nan` use this
-        function.
-        """
-        if array_equivalent(np_array, assert_equal, strict_nan=strict_nan):
-            return
-        raise AssertionError('{0} is not equivalent to {1}.'.format(np_array, assert_equal))
-
-    def assert_categorical_equal(self, res, exp):
-        if not array_equivalent(res.categories, exp.categories):
-            raise AssertionError('categories not equivalent: {0} vs {1}.'.format(res.categories,
-                                                                                 exp.categories))
-        if not array_equivalent(res.codes, exp.codes):
-            raise AssertionError('codes not equivalent: {0} vs {1}.'.format(res.codes,
-                                                                            exp.codes))
-        self.assertEqual(res.ordered, exp.ordered, "ordered not the same")
-        self.assertEqual(res.name, exp.name, "name not the same")
-
-    def assertIs(self, first, second, msg=''):
-        """Checks that 'first' is 'second'"""
-        a, b = first, second
-        assert a is b, "%s: %r is not %r" % (msg.format(a,b), a, b)
-
-    def assertIsNot(self, first, second, msg=''):
-        """Checks that 'first' is not 'second'"""
-        a, b = first, second
-        assert a is not b, "%s: %r is %r" % (msg.format(a,b), a, b)
-
-    def assertIsNone(self, expr, msg=''):
-        """Checks that 'expr' is None"""
-        self.assertIs(expr, None, msg)
-
-    def assertIsNotNone(self, expr, msg=''):
-        """Checks that 'expr' is not None"""
-        self.assertIsNot(expr, None, msg)
-
-    def assertIn(self, first, second, msg=''):
-        """Checks that 'first' is in 'second'"""
-        a, b = first, second
-        assert a in b, "%s: %r is not in %r" % (msg.format(a,b), a, b)
-
-    def assertNotIn(self, first, second, msg=''):
-        """Checks that 'first' is not in 'second'"""
-        a, b = first, second
-        assert a not in b, "%s: %r is in %r" % (msg.format(a,b), a, b)
-
-    def assertIsInstance(self, obj, cls, msg=''):
-        """Test that obj is an instance of cls
-        (which can be a class or a tuple of classes,
-        as supported by isinstance())."""
-        assert isinstance(obj, cls), (
-            "%sExpected object to be of type %r, found %r instead" % (
-                msg, cls, type(obj)))
-
-    def assertNotIsInstance(self, obj, cls, msg=''):
-        """Test that obj is not an instance of cls
-        (which can be a class or a tuple of classes,
-        as supported by isinstance())."""
-        assert not isinstance(obj, cls), (
-            "%sExpected object to be of type %r, found %r instead" % (
-                msg, cls, type(obj)))
-
-    def assertRaises(self, _exception, _callable=None, *args, **kwargs):
-        """ compat with 2.6; assert that an exception is raised """
-        assertRaises(_exception, _callable, *args, **kwargs)
-
-    def assertRaisesRegexp(self, _exception, _regexp, _callable=None, *args, **kwargs):
-        """ Port of assertRaisesRegexp from unittest in Python 2.7 - used in with statement """
-        assertRaisesRegexp(_exception, _regexp, _callable, *args, **kwargs)
-
 # NOTE: don't pass an NDFrame or index to this function - may not handle it
 # well.
 assert_almost_equal = _testing.assert_almost_equal
@@ -193,15 +107,50 @@ def randbool(size=(), p=0.5):
     return rand(*size) <= p
 
 
-def rands(n):
-    choices = string.ascii_letters + string.digits
-    return ''.join(random.choice(choices) for _ in range(n))
+RANDS_CHARS = np.array(list(string.ascii_letters + string.digits),
+                       dtype=(np.str_, 1))
+RANDU_CHARS = np.array(list(u("").join(map(unichr, lrange(1488, 1488 + 26))) +
+                            string.digits), dtype=(np.unicode_, 1))
 
 
-def randu(n):
-    choices = u("").join(map(unichr, lrange(1488, 1488 + 26)))
-    choices += string.digits
-    return ''.join([random.choice(choices) for _ in range(n)])
+def rands_array(nchars, size, dtype='O'):
+    """Generate an array of byte strings."""
+    retval = (choice(RANDS_CHARS, size=nchars * np.prod(size))
+              .view((np.str_, nchars)).reshape(size))
+    if dtype is None:
+        return retval
+    else:
+        return retval.astype(dtype)
+
+
+def randu_array(nchars, size, dtype='O'):
+    """Generate an array of unicode strings."""
+    retval = (choice(RANDU_CHARS, size=nchars * np.prod(size))
+              .view((np.unicode_, nchars)).reshape(size))
+    if dtype is None:
+        return retval
+    else:
+        return retval.astype(dtype)
+
+
+def rands(nchars):
+    """
+    Generate one random byte string.
+
+    See `rands_array` if you want to create an array of random strings.
+
+    """
+    return ''.join(choice(RANDS_CHARS, nchars))
+
+
+def randu(nchars):
+    """
+    Generate one random unicode string.
+
+    See `randu_array` if you want to create an array of random unicode strings.
+
+    """
+    return ''.join(choice(RANDU_CHARS, nchars))
 
 
 def choice(x, size=10):
@@ -222,6 +171,12 @@ def close(fignum=None):
         _close(fignum)
 
 
+def _skip_if_32bit():
+    import nose
+    import struct
+    if struct.calcsize("P") * 8 < 64:
+        raise nose.SkipTest("skipping for 32 bit")
+
 def mplskip(cls):
     """Skip a TestCase instance if matplotlib isn't installed"""
 
@@ -598,6 +553,109 @@ def isiterable(obj):
 def is_sorted(seq):
     return assert_almost_equal(seq, np.sort(np.array(seq)))
 
+
+def assertIs(first, second, msg=''):
+    """Checks that 'first' is 'second'"""
+    a, b = first, second
+    assert a is b, "%s: %r is not %r" % (msg.format(a, b), a, b)
+
+
+def assertIsNot(first, second, msg=''):
+    """Checks that 'first' is not 'second'"""
+    a, b = first, second
+    assert a is not b, "%s: %r is %r" % (msg.format(a, b), a, b)
+
+
+def assertIn(first, second, msg=''):
+    """Checks that 'first' is in 'second'"""
+    a, b = first, second
+    assert a in b, "%s: %r is not in %r" % (msg.format(a, b), a, b)
+
+
+def assertNotIn(first, second, msg=''):
+    """Checks that 'first' is not in 'second'"""
+    a, b = first, second
+    assert a not in b, "%s: %r is in %r" % (msg.format(a, b), a, b)
+
+
+def assertIsNone(expr, msg=''):
+    """Checks that 'expr' is None"""
+    return assertIs(expr, None, msg)
+
+
+def assertIsNotNone(expr, msg=''):
+    """Checks that 'expr' is not None"""
+    return assertIsNot(expr, None, msg)
+
+
+def assertIsInstance(obj, cls, msg=''):
+    """Test that obj is an instance of cls
+    (which can be a class or a tuple of classes,
+    as supported by isinstance())."""
+    assert isinstance(obj, cls), (
+        "%sExpected object to be of type %r, found %r instead" % (
+            msg, cls, type(obj)))
+
+
+def assertNotIsInstance(obj, cls, msg=''):
+    """Test that obj is not an instance of cls
+    (which can be a class or a tuple of classes,
+    as supported by isinstance())."""
+    assert not isinstance(obj, cls), (
+        "%sExpected object to be of type %r, found %r instead" % (
+            msg, cls, type(obj)))
+
+
+def assert_categorical_equal(res, exp):
+    if not array_equivalent(res.categories, exp.categories):
+        raise AssertionError(
+            'categories not equivalent: {0} vs {1}.'.format(res.categories,
+                                                            exp.categories))
+    if not array_equivalent(res.codes, exp.codes):
+        raise AssertionError(
+            'codes not equivalent: {0} vs {1}.'.format(res.codes, exp.codes))
+
+    if res.ordered != exp.ordered:
+        raise AssertionError("ordered not the same")
+
+    if res.name != exp.name:
+        raise AssertionError("name not the same")
+
+
+def assert_numpy_array_equal(np_array, assert_equal):
+    """Checks that 'np_array' is equal to 'assert_equal'
+
+    Note that the expected array should not contain `np.nan`!
+    Two numpy arrays are equal if all
+    elements are equal, which is not possible if `np.nan` is such an element!
+
+    If the expected array includes `np.nan` use
+    `assert_numpy_array_equivalent(...)`.
+    """
+    if np.array_equal(np_array, assert_equal):
+        return
+    raise AssertionError(
+        '{0} is not equal to {1}.'.format(np_array, assert_equal))
+
+
+def assert_numpy_array_equivalent(np_array, assert_equal, strict_nan=False):
+    """Checks that 'np_array' is equivalent to 'assert_equal'
+
+    Two numpy arrays are equivalent if the arrays have equal non-NaN elements,
+     and `np.nan` in corresponding locations.
+
+    If the the expected array does not contain `np.nan`
+    `assert_numpy_array_equivalent` is the similar to
+    `assert_numpy_array_equal()`. If the expected array includes
+    `np.nan` use this
+    function.
+    """
+    if array_equivalent(np_array, assert_equal, strict_nan=strict_nan):
+        return
+    raise AssertionError(
+        '{0} is not equivalent to {1}.'.format(np_array, assert_equal))
+
+
 # This could be refactored to use the NDFrame.equals method
 def assert_series_equal(left, right, check_dtype=True,
                         check_index_type=False,
@@ -743,10 +801,11 @@ def getArangeMat():
 
 # make index
 def makeStringIndex(k=10):
-    return Index([rands(10) for _ in range(k)])
+    return Index(rands_array(nchars=10, size=k))
+
 
 def makeUnicodeIndex(k=10):
-    return Index([randu(10) for _ in range(k)])
+    return Index(randu_array(nchars=10, size=k))
 
 def makeBoolIndex(k=10):
     if k == 1:
@@ -892,7 +951,7 @@ def makeCustomIndex(nentries, nlevels, prefix='#', names=False, ndupe_l=None,
 
     if ndupe_l is None:
         ndupe_l = [1] * nlevels
-    assert (_is_sequence(ndupe_l) and len(ndupe_l) <= nlevels)
+    assert (is_sequence(ndupe_l) and len(ndupe_l) <= nlevels)
     assert (names is None or names is False
             or names is True or len(names) is nlevels)
     assert idx_type is None or \
@@ -1237,6 +1296,7 @@ _network_error_messages = (
 # or this e.errno/e.reason.errno
 _network_errno_vals = (
     101, # Network is unreachable
+    111, # Connection refused
     110, # Connection timed out
     104, # Connection reset Error
     54,  # Connection reset by peer
@@ -1583,7 +1643,7 @@ class _AssertRaisesContextmanager(object):
 
 
 @contextmanager
-def assert_produces_warning(expected_warning=Warning, filter_level="always"):
+def assert_produces_warning(expected_warning=Warning, filter_level="always", clear=None):
     """
     Context manager for running code that expects to raise (or not raise)
     warnings.  Checks that code raises the expected warning and only the
@@ -1610,6 +1670,19 @@ def assert_produces_warning(expected_warning=Warning, filter_level="always"):
     ..warn:: This is *not* thread-safe.
     """
     with warnings.catch_warnings(record=True) as w:
+
+        if clear is not None:
+            # make sure that we are clearning these warnings
+            # if they have happened before
+            # to guarantee that we will catch them
+            if not is_list_like(clear):
+                clear = [ clear ]
+            for m in clear:
+                try:
+                    m.__warningregistry__.clear()
+                except:
+                    pass
+
         saw_warning = False
         warnings.simplefilter(filter_level)
         yield w
@@ -1689,3 +1762,9 @@ def use_numexpr(use, min_elements=expr._MIN_ELEMENTS):
     yield
     expr._MIN_ELEMENTS = oldmin
     expr.set_use_numexpr(olduse)
+
+
+# Also provide all assert_* functions in the TestCase class
+for name, obj in inspect.getmembers(sys.modules[__name__]):
+    if inspect.isfunction(obj) and name.startswith('assert'):
+        setattr(TestCase, name, staticmethod(obj))
